package Fairness.Graphing;

import edu.rit.numeric.XYSeries;

import java.util.Collection;

/**
 * GraphBounds provides a rectangular bounded region over the data in the {@link Graph}'s series.
 *
 * @author Jason Winnebeck
 */
public class GraphBounds {
    private double xLeft;
    private double xRight;
    private double yBottom;
    private double yTop;

    /**
     * Constructs a new GraphBounds.
     *
     * @param xLeft   min for x axis
     * @param xRight  max for x axis
     * @param yBottom min for y axis
     * @param yTop    max for y axis
     */
    public GraphBounds( double xLeft, double xRight, double yBottom, double yTop ) {
        this.xLeft   = xLeft;
        this.xRight  = xRight;
        this.yBottom = yBottom;
        this.yTop    = yTop;
    }

    /**
     * Calculates a GraphBounds based on the extents of the x and y values found in all of the given
     * data series.
     *
     * @param data non-null Collection of 1 or more non-null XYSeries
     *
     * @return non-null GraphBounds providing the extents
     */
    public static GraphBounds calculateBounds( Collection<? extends XYSeries> data ) {
        GraphBounds ret = new GraphBounds( Double.MAX_VALUE, -Double.MAX_VALUE,
                                           Double.MAX_VALUE, -Double.MAX_VALUE );

        for ( XYSeries series : data ) {
            for ( int i = 0; i < series.length(); ++i ) {
                ret.xLeft   = Math.min( ret.xLeft,   series.x( i ) );
                ret.xRight  = Math.max( ret.xRight,  series.x( i ) );
                ret.yBottom = Math.min( ret.yBottom, series.y( i ) );
                ret.yTop    = Math.max( ret.yTop,    series.y( i ) );
            }
        }

        return ret;
    }

    public double getXLeft() {
        return xLeft;
    }

    public double getXRight() {
        return xRight;
    }

    public double getYBottom() {
        return yBottom;
    }

    public double getYTop() {
        return yTop;
    }

    public void setXLeft( double xLeft ) {
        this.xLeft = xLeft;
    }

    public void setXRight( double xRight ) {
        this.xRight = xRight;
    }

    public void setYBottom( double yBottom ) {
        this.yBottom = yBottom;
    }

    public void setYTop( double yTop ) {
        this.yTop = yTop;
    }
}
