package Fairness.Trees;

import Fairness.Nodes.Node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * SimpleTreeManager is a simple tree manager which is used for testing and debugging purposes.
 * It takes only a single stripe and a child thus forming a chain of single striped nodes.
 *
 * @author Jason Winnebeck
 */
public class SimpleTreeManager extends TreeManager {
    private int nextNodeId = 0;

    private Node rootNode = null;

    @Override protected void setUp() {
        super.setUp();

        if ( sim.getParams().getNumStripes() > 1 )
            throw new IllegalArgumentException( "Cannot support > 1 stripe." );
    }

    public Collection<Node> getRootNodes() {
        if ( rootNode != null )
            return Collections.singletonList( rootNode );
        else
            return Collections.emptyList();
    }

    public void rebuildTree() {
        List<Node> nodes = new ArrayList<Node>( getAllNodes() );
        Collections.shuffle( nodes, sim.getRng() );

        Node currParent = null;
        for ( Node next : nodes ) {
            next.resetNetwork();
            if ( currParent != null )
                currParent.makeTreeInfo( 0 ).addChild( next );
            else
                rootNode = next;
            next.makeTreeInfo( 0 ).setParent( currParent );
            currParent = next;
        }
    }

    public int getNextNodeId() {
        return nextNodeId++;
    }
}
