package Fairness;

import Fairness.Nodes.Node;
import Fairness.Nodes.NodeBehaviors.SimpleNodeBehavior;
import Fairness.Nodes.NodePacket;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 * NodeRoutingTest
 *
 * @author Jason Winnebeck
 */
public class NodeRoutingTest {
    @Test public void testRouting() {
        NodeMonitor monitor = new NodeMonitor();
        MonitoringBehavior b = new MonitoringBehavior();

        Simulation sim = new Simulation( new SimulationParameters( 2, 100, 1, 100.0, 0 ) );

        Node parent = new Node( sim, 0, 2000, 2000, monitor, b, new NoTreeBehavior() );

        Node child1 = new Node( sim, 1, 2000, 2000, monitor, b, new NoTreeBehavior() );
        Node child2 = new Node( sim, 2, 2000, 2000, monitor, b, new NoTreeBehavior() );
        parent.makeTreeInfo( 0 ).addChild( child1 );
        child1.makeTreeInfo( 0 ).setParent( parent );
        parent.makeTreeInfo( 0 ).addChild( child2 );
        child2.makeTreeInfo( 0 ).setParent( parent );

        Node child3 = new Node( sim, 1, 2000, 2000, monitor, b, new NoTreeBehavior() );
        parent.makeTreeInfo( 1 ).addChild( child3 );
        child3.makeTreeInfo( 1 ).setParent( parent );

        b.setExpected( null, parent, 0 );
        parent.processPacket( sim, new NodePacket( null, 0, false, true, 1 ) );
        assertFalse( b.expected );

        //We should be getting just two events
        b.setExpected( parent, child1, 0 );
        assertTrue( sim.next() );
        assertFalse( b.expected );

        b.setExpected( parent, child2, 0 ); //TODO: Maybe this can get reversed?
        assertFalse( sim.next() );
        assertFalse( b.expected );

        //Send on the other stripe
        b.setExpected( null, parent, 1 );
        parent.processPacket( sim, new NodePacket( null, 1, false, true, 1 ) );
        assertFalse( b.expected );

        b.setExpected( parent, child3, 1 );
        assertFalse( sim.next() );
        assertFalse( b.expected );
    }

    private static class MonitoringBehavior extends SimpleNodeBehavior {
        public boolean expected = false;
        public Node expectedMyNode;
        public Node expectedSource;
        public int expectedStripe;

        public void receivedPacket( Node myNode, NodePacket packet ) {
            assertTrue( expected );
            assertEquals( expectedMyNode, myNode );
            assertEquals( expectedStripe, packet.getStripeId() );
            assertEquals( expectedSource, packet.getSource() );

            expected = false;
        }

        public void setExpected( Node src, Node dest, int stripe ) {
            expected = true;
            expectedMyNode = dest;
            expectedSource = src;
            expectedStripe = stripe;
        }
    }
}
